@@ -57,6 +57,14 @@ unless (@ARGV) {
     die "usage: $0 language [path]\n";
 }
 
+if ($ARGV[0] eq '--help') {
+    print <<HELP;
+usage: $0 language [PATH]
+Use perldoc $0 to read the documentation.
+HELP
+    exit 0;
+}
+
 ##  determine the language we're trying to build
 my $lang = $ARGV[0];
 my $lclang = lc $lang;
